@@ -143,7 +143,7 @@ class Auth(object):
     The passwords hashes are stored with the following
     format (without the spaces):
         
-        dsha512 $ cost $ salt $ salted_and_hashed_password
+        sha512 $ cost $ salt $ salted_and_hashed_password
         - or -
         bcrypt $ cost $ salt salted_and_hashed_password
 
@@ -193,7 +193,7 @@ class Auth(object):
                 settings.hash_alg = 'bcrypt'
             else:
                 self.hash_func = hash_sha512
-                settings.hash_alg = 'sha512'
+                settings.hash_alg = 'dsha512'
         
         elif hash_alg == 'scrypt':
             if not scrypt:
@@ -205,15 +205,15 @@ class Auth(object):
                 raise ImportError("Hash algorithm 'bcrypt' not available")
             self.hash_func = hash_bcrypt
         
-        elif hash_alg == 'sha512':
+        elif hash_alg == 'dsha512':
             self.hash_func = hash_sha512
         
-        elif hash_alg == 'sha256':
+        elif hash_alg == 'dsha256':
             self.hash_func = hash_sha256
         
         else:
             raise AssertionError("The `hash_alg` parameter must be either " \
-                "'scrypt', 'bcrypt', 'sha512' or 'sha256'")
+                "'scrypt', 'bcrypt', 'dsha512' or 'dsha256'")
         
         hash_cost = settings.hash_cost or 4
         assert isinstance(hash_cost, (int, long)) and hash_cost >= 4, \
@@ -314,6 +314,9 @@ class Auth(object):
             must_update_hash = (hash_alg != self.settings.hash_alg)
             must_update_cost = (hash_cost != self.settings.hash_cost)
             if must_update_hash or must_update_cost:
+                print '\n------------ Must update! -------------\n'
+                print hash_alg, self.settings.hash_alg
+                print hash_cost, self.settings.hash_cost
                 user.password = self.hash_password(password)
         return user
 
@@ -339,9 +342,9 @@ class Auth(object):
         
         return user
 
-    def hash_password(self, password):
+    def hash_password(self, password, salt=None):
         password += self.settings.pepper
-        hashed = self.hash_func(password, cost=self.settings.hash_cost)
+        hashed = self.hash_func(password, salt=salt, cost=self.settings.hash_cost)
         return hashed
 
     def check_password(self, password, stored_password):
@@ -429,7 +432,6 @@ class Auth(object):
         self.app.session_interface.invalidate(request)
         request.user = None
 
-
     def create_user(self, login, passw, **data):
         """[-login] USER_NAME [-passw] PASSWORD
         Creates a new user.
@@ -464,14 +466,13 @@ class Auth(object):
     def sign_in_view(self, request, **kwargs):
         """Signs a user in.
         """
-        redirect_url = request.session.get(REDIRECT_FIELD_NAME)
+        redirect_url = request.session.pop(REDIRECT_FIELD_NAME, None)
         redirect_url = redirect_url or self.settings.sign_in_redirect
         if callable(redirect_url):
             redirect_url = redirect_url()
         
         # Redirect if there's already a signed user
         if request.user:
-            request.session.pop(REDIRECT_FIELD_NAME, None)
             return redirect_to(redirect_url)
         
         credentials = request.form
