@@ -113,7 +113,7 @@ def hash_sha512(data, salt=None, cost=DEFAULT_COST, keylen=24):
     return 'dsha512$%i$%s$%s' % (cost, salt, hashed)
 
 
-def hash_bcrypt(data, hashed=None, cost=DEFAULT_COST):
+def hash_bcrypt(data, hashed=None, cost=DEFAULT_COST, salt=None):
     """OpenBSD Blowfish password hashing.
     
     Requires py-bcrypt <http://pypi.python.org/pypi/py-bcrypt/>
@@ -132,7 +132,7 @@ def hash_bcrypt(data, hashed=None, cost=DEFAULT_COST):
     if hashed:
         salt = hashed.replace('bcrypt$', '$2a$', 1)
     else:
-        salt = bcrypt.gensalt(cost)
+        salt = salt or bcrypt.gensalt(cost)
     hashed = bcrypt.hashpw(data, salt)
     return hashed.replace('$2a$', 'bcrypt$', 1)
 
@@ -181,6 +181,7 @@ def hash_old_sha512(data, salt=None, cost=DEFAULT_COST):
 def get_user_hmac(user):
     # This use as a seed a few chars from the password SALT.
     # NOT from the real password.
+    print '\n', user.password, '\n'
     mac = hashlib.sha256(user.password[10:18])
     mac = mac.hexdigest()[:8]
     return '%s$%s' % (user.id, mac)
